function joboptions=make_cluster_options(machineName,localDirectory,remoteDirectory,exeCmd,numCPUs,jobID,sharedFileSystem,singleMachine,checkType,checkFile,checkToken,checkCommandFile,wallClockHours,jobScriptName,sshPort)

% Script to make an options structure for submitting jobs to a computing cluster. To be 
% used in conjunction with the ClusterTools toolbox. You will need to modify this file 
% to work on your own hardware. It is only provided as a guide. Parts that you can/must 
% change are indicated below.

if nargin==0
  disp(['Supported machineNames: '])
  disp(['rzcluster, fats, lonestar, ranger']) % CHANGE THIS IF YOU ADD YOUR OWN MACHINES
  return
end

% Some argument checking
if ~strcmp(machineName,'default')
  if nargin<5
    error('Must specify at least 5 arguments')
  end
  if isempty(exeCmd)
    error('Must specify the execution command')
  end
  if isempty(numCPUs)
    error('Must specify a value for number of CPUs')
  end
else
  if nargin<2
    localDirectory=[];
  end
  if nargin<3
    remoteDirectory=[];
  end
  if nargin<4
    exeCmd=[];
  end
  if nargin<5
    numCPUs=[];
  end
end

% Default user to email job information to. YOU MUST CHANGE THIS TO YOUR OWN E-MAIL ADDRESS
email='me@nowhere.edu'; 

% Defaults. Best not to modify these.
if nargin<6 || isempty(jobID)
  jobID='MyJob';
end

if nargin<7 || isempty(sharedFileSystem)
  sharedFileSystem=0;
else
  if ~ismember(sharedFileSystem,[0 1])
    error('ERROR: sharedFileSystem must be 0 or 1')
  end
end  

if nargin<8 || isempty(singleMachine)
  singleMachine=0;
else
  if ~ismember(singleMachine,[0 1])
    error('ERROR: singleMachine must be 0 or 1')
  end
end
if singleMachine
  if ~strcmp(localDirectory,remoteDirectory)
    error('ERROR: Local and remote paths differ even though singleMachine is set to 1')
  end
  if ~sharedFileSystem
    error('ERROR: singleMachine is set to 1, but sharedFileSystem is set to 0')
  end    
end

if nargin<9 || isempty(checkType)
  checkType=1;
else
  if ~ismember(checkType,[1 2])
    error('ERROR: checkType must be 1 or 2')
  end
end

if nargin<10 || isempty(checkFile)
  if checkType==1
    checkFile='donefile';
  elseif checkType==2
    checkFile='logfile';
  end
end  

if nargin<11 && (checkType==2)
  error('Must specify checkToken')
end    

if nargin<12 || isempty(checkCommandFile)
  checkCommandFile=[jobID '_checkjob'];
end

if nargin<13 || isempty(wallClockHours)
  wallClockHours=24;
end
wallClockHours=ceil(wallClockHours);

if nargin<14 || isempty(jobScriptName)
  jobScriptName='jobscript';
end

if nargin<15
  sshPort=[];
end

stdout=[jobID '.stdout'];
stderr=[jobID '.stderr'];
% done setting defaults

% Common options
joboptions.machine=machineName;
joboptions.remdir=remoteDirectory;
joboptions.locdir=localDirectory;  
joboptions.execmd=exeCmd;
joboptions.jobid=jobID;
joboptions.scriptname=jobScriptName;
joboptions.sharedfilesystem=sharedFileSystem;
joboptions.singlemachine=singleMachine;

% Options to check for job completion. Currently only 2 methods are supported (see below).
% YOU CAN ADD MORE IF YOU LIKE. IF YOU DO, YOU WILL ALSO NEED TO MODIFY jobsubmit.m and 
% jobcheck.m.
joboptions.checktype=checkType;
if checkType==1 % use a remote script to test for presence of a file to check for job completion
  joboptions.checkfile=checkFile;
elseif checkType==2 % test for presence of a string in a file to check for job completion
  joboptions.checkfile=checkFile;
  if isempty(checkToken)
    error('Must specify checkToken')
  end    
  joboptions.checktoken=checkToken;
end
joboptions.checkcommandfile=checkCommandFile;

if ~isempty(sshPort)
  joboptions.sshport=sshPort;
end
% Done setting common options

% Machine specific options. YOU CAN MODIFY/ADD YOUR OWN MACHINES TO THE LIST BELOW.
switch machineName
case 'bigmachine1'
  joboptions.system='PBS';
  joboptions.frontend='me@bigmachine1.gov';
  queue='normal';
  wallclocktime=['walltime=' int2str(wallClockHours) ':00:00'];  
  if rem(numCPUs,8)
    error('Number of CPUs must be divisible by 8')
  end
  numNodes=numCPUs/8;
  nodes=['select=' int2str(numNodes) ':normal=true:ncpus=8:mpiprocs=8'];
  joboptions.numcpus=numCPUs;
  joboptions.subscript{1}=['#!/bin/bash'];  
  joboptions.subscript{2}=['#PBS -o ' stdout];
  joboptions.subscript{3}=['#PBS -e ' stderr];
  joboptions.subscript{4}=['#PBS -l ' nodes];
  joboptions.subscript{5}=['#PBS -M ' email];
  joboptions.subscript{6}=['#PBS -q ' queue];
  joboptions.subscript{7}=['#PBS -N ' jobID];
  joboptions.subscript{8}=['#PBS -l ' wallclocktime];    

  joboptions.extracmds{1}=['cd $PBS_O_WORKDIR'];
  joboptions.extracmds{2}='. /usr/local/Modules/3.2.6/init/bash';
  joboptions.extracmds{3}='module load openmpi-gnu';
  joboptions.extracmds{4}='export TMPDIR=''.''';
  
  joboptions.subcmd='qsub'; % job submission command
  
case 'bigmachine2'
  joboptions.system='LSF';	
  joboptions.frontend='me@bigmachine2.gov';
  queue='normal';
  if rem(numCPUs,4)
    error('Number of CPUs must be divisible by 4')
  end
  numNodes=numCPUs/4;  
  nodes=int2str(numCPUs); % number of CPUs
  joboptions.numcpus=numCPUs;;      
  wallclocktime=[int2str(wallClockHours) ':00'];
  
  joboptions.subscript{1}=['#!/bin/csh'];
  joboptions.subscript{2}=['#BSUB -o ' stdout];
  joboptions.subscript{3}=['#BSUB -e ' stderr];
  joboptions.subscript{4}=['#BSUB -n ' nodes];
  joboptions.subscript{5}=['#BSUB -q ' queue];
  joboptions.subscript{6}=['#BSUB -J ' jobID];
  joboptions.subscript{7}=['#BSUB -W ' wallclocktime];  
  joboptions.subscript{8}=['#BSUB -R ' '''span[ptile=4]'''];

  joboptions.extracmds{1}=['set echo'];
  joboptions.extracmds{2}=['cd ' remoteDirectory];
  
  joboptions.subcmd='bsub <'; % job submission command
  
case 'default'
% default
  joboptions.system='PBS';
  joboptions.frontend=[];
  joboptions.numcpus=numCPUs;
  joboptions.subscript{1}=[];  
  joboptions.extracmds{1}=[];
  joboptions.subcmd=[]; % job submission command

otherwise
  error(['ERROR: Unknown machineName'])  
end

